unit uDefs;

interface

const
  VIDE = 0;
  JAUNE = 1;
  ROUGE = -1;
  GAGNANT = 2;

  NB_ALIGN = 4;
  MAX_COL = 6;
  MAX_ROW = 5;

  PROF_MAX = 5;

type
  TGrille =  array[0..MAX_COL, 0..MAX_ROW] of ShortInt;
  TScore = array[ROUGE..JAUNE] of ShortInt;
  TJeton = record
    X, Y: ShortInt;
    Joueur: ShortInt;
  end;

var
  NbManches: ShortInt;

  function ConfirmerAbandon: Boolean;
  procedure MsgColPleine;
  procedure MsgFinManche(const Joueur: ShortInt; var Score: TScore);

  function DispoCol(const G: TGrille; const ACol: Byte): Byte;
  function DispoGrille(const G: TGrille): Byte;
  function PointDeChute(const G: TGrille; const ACol: Byte): Byte;
  function JoueurGagnant(var G: TGrille; const Joueur: ShortInt): Boolean;
  function MancheEnCours(const G: TGrille): Boolean;

implementation

uses
  Dialogs;

//------------------------------------------------------------------------------
// Confirmer l'abandon de la partie
//------------------------------------------------------------------------------
function ConfirmerAbandon: Boolean;
const
  IDYES = 6;
begin
  result := (MessageDlg('Abandonner cette manche ?', mtConfirmation,
                        [mbYes, mbNo], 0) = IDYES);
end;
//------------------------------------------------------------------------------
// Message 'Colonne pleine'
//------------------------------------------------------------------------------
procedure MsgColPleine;
begin
  MessageDlg('Colonne pleine !', mtWarning, [mbOK], 0);
end;
//------------------------------------------------------------------------------
// Message 'Fin de manche / Fin de partie'
//------------------------------------------------------------------------------
procedure MsgFinManche(const Joueur: ShortInt; var Score: TScore);
var
  msg: string;
begin
  if Joueur = VIDE then
    msg := 'Match nul !'
  else
  begin
    if Score[Joueur] >= NbManches then
      begin
        if Joueur = JAUNE then
          msg := 'Les Jaunes ont gagn la partie !'
        else
          msg := 'Les Rouges ont gagn la partie !';
        Score[JAUNE] := 0;
        Score[ROUGE] := 0;
      end
    else
      begin
        if Joueur = JAUNE then
          msg := 'Les Jaunes gagnent cette manche.'
        else
          msg := 'Les Rouges gagnent cette manche.';
      end;
  end;
  MessageDlg(msg, mtInformation, [mbOK], 0);
end;
//------------------------------------------------------------------------------
// Vrifier si une manche est en cours
//------------------------------------------------------------------------------
function MancheEnCours(const G: TGrille): Boolean;
begin
  result := DispoGrille(G) < ((MAX_COL + 1) * (MAX_ROW + 1));
end;
//------------------------------------------------------------------------------
// Calcul du nombre d'emplacements vides dans la colonne
//------------------------------------------------------------------------------
function DispoCol(const G: TGrille; const ACol: Byte): Byte;
var
  Row: Byte;
begin
  result := 0;
  for Row := 0 to MAX_ROW do
    if G[ACol, Row] = VIDE then Inc(Result);
end;
//------------------------------------------------------------------------------
// Calcul du nombre d'emplacement vides dans la grille
//------------------------------------------------------------------------------
function DispoGrille(const G: TGrille): Byte;
var
  Col: Byte;
begin
  result := 0;
  for Col := 0 to MAX_COL do result := result + DispoCol(G, Col);
end;
//------------------------------------------------------------------------------
// Calcul du point de chute d'un jeton dans une colonne
//------------------------------------------------------------------------------
function PointDeChute(const G: TGrille; const ACol: Byte): Byte;
begin
  result := DispoCol(G, ACol) - 1;
end;

//------------------------------------------------------------------------------
// Teste la victoire du joueur
//------------------------------------------------------------------------------
function JoueurGagnant(var G: TGrille; const Joueur: ShortInt): Boolean;
var
  Col, Row: Byte;
  //-----------------------------------------------------------------
  function DirectionGagnante(const dX, dY: ShortInt): Boolean;
  var
    i, X, Y: ShortInt;
  begin
    X := Col + (NB_ALIGN - 1) * dX;
    Y := Row + (NB_ALIGN - 1) * dY;
    if (X < 0) or (X > MAX_COL) or (Y < 0) or (Y > MAX_ROW) then
      result := False
    else begin
      i := 0;
      while (i < NB_ALIGN) and (G[Col + i * dX, Row + i * dY] = Joueur) do
        Inc(i);
      result := (i = NB_ALIGN);
      if result then
        for i := 0 to NB_ALIGN - 1 do G[Col + i * dX, Row + i * dY] := GAGNANT;
    end;
  end;
  //-----------------------------------------------------------------
begin
  result := False;

  Col := 0;
  while (Col <= MAX_COL) and (not result) do
  begin
    Row := 0;
    while (Row <= MAX_ROW) and (not Result) do
    begin
      result := (DirectionGagnante(1, 0) or // Horizontal
                 DirectionGagnante(0, 1) or // Vertical
                 DirectionGagnante(1, 1) or // Diagonal \
                 DirectionGagnante(1,-1));  // Diagonal /
      Inc(Row);
    end;
    Inc(Col);
  end;
end;

end.
