unit uIA;

{$I P4.inc}

interface

uses
  uDefs{$IFDEF DEBUG}, SysUtils, Dialogs, uMain{$ENDIF};

  function DecisionMinMax(G: TGrille; pMax: ShortInt): ShortInt;

implementation

//------------------------------------------------------------------------------
// Appliquer le coup d'un joueur dans une colonne
//------------------------------------------------------------------------------
function AppliquerCoup(G: TGrille; Joueur: ShortInt; Col: Byte): TGrille;
begin
  G[Col, PointDeChute(G, Col)] := Joueur;
  result := G;
end;
//------------------------------------------------------------------------------
// Evaluation d'une position pour un joueur
//------------------------------------------------------------------------------
function EvalPos(G: TGrille; Joueur: ShortInt; const Col, Row: Byte): Integer;
  //------------------------------------------------------
  // Evaluation dans une direction
  //------------------------------------------------------
  function EvalPosDirection(const dX, dY: ShortInt): Integer;
  var
    i, X, Y: ShortInt;
    Opposant: ShortInt;
  begin
    result := 0;

    X := Col + (NB_ALIGN - 1) * dX;
    Y := Row + (NB_ALIGN - 1) * dY;
    if (X < 0) or (X > MAX_COL) or (Y < 0) or (Y > MAX_ROW) then Exit;

    if (Joueur = ROUGE) then
      Opposant := JAUNE
    else
      Opposant := ROUGE;

    i := 0;
    while (i < NB_ALIGN) and (G[Col + i * dX, Row + i * dY] <> Opposant) do
      Inc(i);
    if (i = NB_ALIGN) then result := 1;
  end;
  //------------------------------------------------------
begin
  result := EvalPosDirection(1, 0) +  // Horizontal
            EvalPosDirection(0, 1) +  // Vertical
            EvalPosDirection(1, 1) +  // Diagonal \
            EvalPosDirection(1,-1);   // Diagonal /
end;
//------------------------------------------------------------------------------
// Evaluation de la grille pour un joueur
//------------------------------------------------------------------------------
function EvalGrille(G: TGrille; Joueur: ShortInt) : integer;
var
  Col, Row: Byte;
begin
  result := 0;
  for Col := 0 to MAX_COL do
    for Row := 0 to MAX_ROW do
      result := result + EvalPos(G, Joueur, Col, Row);
end;
//------------------------------------------------------------------------------
// Evaluation heuristique d'une situation pour les rouges
//------------------------------------------------------------------------------
function FH(G: TGrille): Integer;
begin
  result := EvalGrille(G, ROUGE) - EvalGrille(G, JAUNE);
end;
//------------------------------------------------------------------------------
// Evalue une situation de jeu pour les rouges suivant l'algorithme MinMax
//------------------------------------------------------------------------------
function ValeurMinMax(G: TGrille; Profondeur: ShortInt; EtatMax: Boolean): Integer;
var
  Col, Valeur: integer;
begin
  if JoueurGagnant(G, ROUGE) then
    result := MAXINT
  else if JoueurGagnant(G, JAUNE) then
    result := -MAXINT
  else if Profondeur = 0 then
    result := FH(G) // Fonction heuristique: valuation de la feuille
  else begin
    if EtatMax then begin
      result := -MAXINT;
      for Col := 0 to MAX_COL do
      begin
        if (DispoCol(G, Col) <> 0) then begin
          Valeur := ValeurMinMax(AppliquerCoup(G, ROUGE, Col), Profondeur - 1, False);
          if (Valeur > result) then
            result := Valeur;
        end;
      end
    end else begin
      result := MAXINT;
      for Col := 0 to MAX_COL do begin
        if (DispoCol(G, Col) <> 0) then begin
          Valeur := ValeurMinMax(AppliquerCoup(G, JAUNE, Col), Profondeur - 1, True);
          if (Valeur < result) then
            result := Valeur;
        end;
      end;
    end;
  end;

{$IFDEF DEBUG}
  FormMain.Memo1.Lines.Add('Prof: ' + IntToStr(Profondeur) +
                           ', EtatMax: ' + BoolToStr(EtatMax, True) +
                           ', Valeur: ' + IntToStr(result));
{$ENDIF}
end;

//------------------------------------------------------------------------------
// Dcider du coup  jouer pour les rouges
//------------------------------------------------------------------------------
function DecisionMinMax(G: TGrille; pMax: ShortInt): ShortInt;
var
  Col: Byte;
  Valeur, ValeurMax: Integer;
begin
  result := -1;
  ValeurMax := -MAXINT;

{$IFDEF DEBUG}
  with FormMain.Memo1.Lines do
  begin
    BeginUpdate;
    Clear;
  end;
{$ENDIF}

  for Col := 0 to MAX_COL do
  begin
    if (DispoCol(G, Col) <> 0) then
    begin
      Valeur := ValeurMinMax(AppliquerCoup(G, ROUGE, Col), pMax, False);
      if (Valeur >= ValeurMax) then
      begin
        ValeurMax := Valeur;
        result := Col;
      end;
{$IFDEF DEBUG}
      with FormMain.Memo1.Lines do
      begin
        Add('Col: ' + IntToStr(Col) + ', Valeur: ' + IntToStr(Valeur));
        Add('*************************');
      end;
{$ENDIF}
    end;
  end;
{$IFDEF DEBUG}
  with FormMain.Memo1.Lines do
  begin
    Add('Jou en Col: ' + IntToStr(result));   
    EndUpdate;
  end;
{$ENDIF}

  if result = -1 then
  begin
{$IFDEF DEBUG}
    MessageDlg('Retour MinMax = -1', mtError, [mbOK], 0);
{$ENDIF}
  // Choisir une colonne de facon alatoire
    result := Random(MAX_COL + 1);
    while DispoCol(G, result) = 0 do result := Random(MAX_COL + 1);
  end;
end;

initialization
  Randomize;

end.
